@@ -298,18 +298,18 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
             ),
         )
 
-    for keyword in [
-        "properties", "additionalProperties", "unevaluatedProperties",
-    ]:
-        if keyword in schema:
-            schema_value = schema[keyword]
-            if validator.is_type(schema_value, "boolean") and schema_value:
-                evaluated_keys += instance.keys()
+    properties = schema.get("properties")
+    if validator.is_type(properties, "object"):
+        evaluated_keys += properties.keys() & instance.keys()
 
-            elif validator.is_type(schema_value, "object"):
-                for property in schema_value:
-                    if property in instance:
-                        evaluated_keys.append(property)
+    for keyword in ["additionalProperties", "unevaluatedProperties"]:
+        if (subschema := schema.get(keyword)) is None:
+            continue
+        evaluated_keys += (
+            key
+            for key, value in instance.items()
+            if is_valid(validator.descend(value, subschema))
+        )
 
     if "patternProperties" in schema:
         for property in instance:
@@ -326,13 +326,12 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
             )
 
     for keyword in ["allOf", "oneOf", "anyOf"]:
-        if keyword in schema:
-            for subschema in schema[keyword]:
-                errs = next(validator.descend(instance, subschema), None)
-                if errs is None:
-                    evaluated_keys += find_evaluated_property_keys_by_schema(
-                        validator, instance, subschema,
-                    )
+        for subschema in schema.get(keyword, []):
+            if not is_valid(validator.descend(instance, subschema)):
+                continue
+            evaluated_keys += find_evaluated_property_keys_by_schema(
+                validator, instance, subschema,
+            )
 
     if "if" in schema:
         if validator.evolve(schema=schema["if"]).is_valid(instance):
@@ -349,3 +348,8 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
             )
 
     return evaluated_keys
+
+
+def is_valid(errs_it):
+    """Whether there are no errors in the given iterator."""
+    return next(errs_it, None) is None
